@@ -89,13 +89,13 @@ def clientTestCmd(argv):
 
     #open synchronisation FIFO
     synchro = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    synchro.settimeout(5)
+    synchro.settimeout(15)
     synchro.connect((address[0], address[1]-1))
 
     def connect():
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         if hasattr(sock, 'settimeout'): #It's a python 2.3 feature
-            sock.settimeout(5)
+            sock.settimeout(15)
         sock.connect(address)
         c = TLSConnection(sock)
         return c
@@ -126,6 +126,32 @@ def clientTestCmd(argv):
 
     test_no += 1
 
+    print("Test {0} - good X.509/w RSA-PSS cert".format(test_no))
+    synchro.recv(1)
+    connection = connect()
+    connection.handshakeClientCert(serverName=address[0])
+    testConnClient(connection)
+    assert(isinstance(connection.session.serverCertChain, X509CertChain))
+    assert(connection.session.serverName == address[0])
+    assert(connection.session.cipherSuite in constants.CipherSuite.aeadSuites)
+    assert(connection.encryptThenMAC == False)
+    connection.close()
+
+    test_no += 1
+
+    print("Test {0} - good X.509/w RSA-PSS sig".format(test_no))
+    synchro.recv(1)
+    connection = connect()
+    connection.handshakeClientCert(serverName=address[0])
+    testConnClient(connection)
+    assert(isinstance(connection.session.serverCertChain, X509CertChain))
+    assert(connection.session.serverName == address[0])
+    assert(connection.session.cipherSuite in constants.CipherSuite.aeadSuites)
+    assert(connection.encryptThenMAC == False)
+    connection.close()
+
+    test_no += 1
+
     print("Test {0} - good X509, SSLv3".format(test_no))
     synchro.recv(1)
     connection = connect()
@@ -195,7 +221,7 @@ def clientTestCmd(argv):
 
     test_no += 1
 
-    print("Test {0} - good SRP".format(test_no))
+    print("Test {0} - good SRP (db)".format(test_no))
     synchro.recv(1)
     connection = connect()
     connection.handshakeClientSRP("test", "password")
@@ -204,7 +230,7 @@ def clientTestCmd(argv):
 
     test_no += 1
 
-    print("Test {0} - good SRP (db)".format(test_no))
+    print("Test {0} - good SRP".format(test_no))
     synchro.recv(1)
     connection = connect()
     connection.handshakeClientSRP("test", "password")
@@ -702,7 +728,7 @@ def clientTestCmd(argv):
         test_no += 1
 
         print("Test {0}: POP3 good".format(test_no))
-    except socket.error as e:
+    except (socket.error, socket.timeout) as e:
         print("Non-critical error: socket error trying to reach internet server: ", e)   
 
     synchro.close()
@@ -757,6 +783,20 @@ def serverTestCmd(argv):
     s = open(os.path.join(dir, "serverX509Key.pem")).read()
     x509Key = parsePEMKey(s, private=True)
 
+    with open(os.path.join(dir, "serverRSAPSSSigCert.pem")) as f:
+        x509CertRSAPSSSig = X509().parse(f.read())
+    x509ChainRSAPSSSig = X509CertChain([x509CertRSAPSSSig])
+    with open(os.path.join(dir, "serverRSAPSSSigKey.pem")) as f:
+        x509KeyRSAPSSSig = parsePEMKey(f.read(), private=True)
+
+    with open(os.path.join(dir, "serverRSAPSSCert.pem")) as f:
+        x509CertRSAPSS = X509().parse(f.read())
+    x509ChainRSAPSS = X509CertChain([x509CertRSAPSS])
+    assert x509CertRSAPSS.certAlg == "rsa-pss"
+    with open(os.path.join(dir, "serverRSAPSSKey.pem")) as f:
+        x509KeyRSAPSS = parsePEMKey(f.read(), private=True,
+                                    implementations=["python"])
+
     test_no = 0
 
     print("Test {0} - Anonymous server handshake".format(test_no))
@@ -779,6 +819,30 @@ def serverTestCmd(argv):
 
     test_no += 1
 
+    print("Test {0} - good X.509/w RSA-PSS sig".format(test_no))
+    synchro.send(b'R')
+    connection = connect()
+    connection.handshakeServer(certChain=x509ChainRSAPSSSig,
+                               privateKey=x509KeyRSAPSSSig)
+    assert(connection.session.serverName == address[0])
+    assert(connection.extendedMasterSecret)
+    testConnServer(connection)
+    connection.close()
+
+    test_no += 1
+
+    print("Test {0} - good X.509/w RSA-PSS cert".format(test_no))
+    synchro.send(b'R')
+    connection = connect()
+    connection.handshakeServer(certChain=x509ChainRSAPSS,
+                               privateKey=x509KeyRSAPSS)
+    assert(connection.session.serverName == address[0])
+    assert(connection.extendedMasterSecret)
+    testConnServer(connection)
+    connection.close()
+
+    test_no += 1
+
     print("Test {0} - good X.509, SSL v3".format(test_no))
     synchro.send(b'R')
     connection = connect()
@@ -845,20 +909,6 @@ def serverTestCmd(argv):
 
     test_no += 1
 
-    print("Test {0} - good SRP".format(test_no))
-    verifierDB = VerifierDB()
-    verifierDB.create()
-    entry = VerifierDB.makeVerifier("test", "password", 1536)
-    verifierDB[b"test"] = entry
-
-    synchro.send(b'R')
-    connection = connect()
-    connection.handshakeServer(verifierDB=verifierDB)
-    testConnServer(connection)
-    connection.close()
-
-    test_no += 1
-
     print("Test {0} - good SRP (db)".format(test_no))
     try:
         (db_file, db_name) = mkstemp()
@@ -876,7 +926,26 @@ def serverTestCmd(argv):
         testConnServer(connection)
         connection.close()
     finally:
-        os.remove(db_name)
+        try:
+            os.remove(db_name)
+        except FileNotFoundError:
+            # dbm module may create files with different names depending on
+            # platform
+            os.remove(db_name + ".dat")
+
+    test_no += 1
+
+    print("Test {0} - good SRP".format(test_no))
+    verifierDB = VerifierDB()
+    verifierDB.create()
+    entry = VerifierDB.makeVerifier("test", "password", 1536)
+    verifierDB[b"test"] = entry
+
+    synchro.send(b'R')
+    connection = connect()
+    connection.handshakeServer(verifierDB=verifierDB)
+    testConnServer(connection)
+    connection.close()
 
     test_no += 1
 
