@@ -5,6 +5,7 @@
 
 import sys
 import os
+import platform
 import math
 import binascii
 import traceback
@@ -19,7 +20,13 @@ if sys.version_info >= (3,0):
     # So, python 2.6 requires strings, python 3 requires 'bytes',
     # and python 2.7 can handle bytearrays...     
     def compatHMAC(x): return bytes(x)
-    
+
+    def compatAscii2Bytes(val):
+        """Convert ASCII string to bytes."""
+        if isinstance(val, str):
+            return bytes(val, 'ascii')
+        return val
+
     def raw_input(s):
         return input(s)
     
@@ -65,11 +72,17 @@ else:
     # Python 2.6 requires strings instead of bytearrays in a couple places,
     # so we define this function so it does the conversion if needed.
     # same thing with very old 2.7 versions
-    if sys.version_info < (2, 7) or sys.version_info < (2, 7, 4):
+    # or on Jython
+    if sys.version_info < (2, 7) or sys.version_info < (2, 7, 4) \
+            or platform.system() == 'Java':
         def compat26Str(x): return str(x)
     else:
         def compat26Str(x): return x
 
+    def compatAscii2Bytes(val):
+        """Convert ASCII string to bytes."""
+        return val
+
     # So, python 2.6 requires strings, python 3 requires 'bytes',
     # and python 2.7 can handle bytearrays...     
     def compatHMAC(x): return compat26Str(x)
