@@ -260,5 +260,29 @@ class TestHandshakeSettings(unittest.TestCase):
         with self.assertRaises(ValueError):
             hs.validate()
 
+    def test_invalid_dhParams(self):
+        hs = HandshakeSettings()
+        hs.dhParams = (2, 'bd')
+        with self.assertRaises(ValueError):
+            hs.validate()
+
+    def test_invalid_dhGroups(self):
+        hs = HandshakeSettings()
+        hs.dhGroups = ["ffdhe2048", "ffdhe1024"]
+        with self.assertRaises(ValueError):
+            hs.validate()
+
+    def test_invalid_rsaScheme(self):
+        hs = HandshakeSettings()
+        hs.rsaSchemes += ["rsassa-pkcs1-1_5"]
+        with self.assertRaises(ValueError):
+            hs.validate()
+
+    def test_invalid_defaultCurve_name(self):
+        hs = HandshakeSettings()
+        hs.defaultCurve = "ffdhe2048"
+        with self.assertRaises(ValueError):
+            hs.validate()
+
 if __name__ == '__main__':
     unittest.main()
