@@ -10,12 +10,12 @@ except ImportError:
 from tlslite.messages import ClientHello, ServerHello, RecordHeader3, Alert, \
         RecordHeader2, Message, ClientKeyExchange, ServerKeyExchange, \
         CertificateRequest, CertificateVerify, ServerHelloDone, ServerHello2, \
-        ClientMasterKey, ClientFinished, ServerFinished
+        ClientMasterKey, ClientFinished, ServerFinished, CertificateStatus
 from tlslite.utils.codec import Parser
 from tlslite.constants import CipherSuite, CertificateType, ContentType, \
         AlertLevel, AlertDescription, ExtensionType, ClientCertificateType, \
         HashAlgorithm, SignatureAlgorithm, ECCurveType, GroupName, \
-        SSL2HandshakeType
+        SSL2HandshakeType, CertificateStatusType
 from tlslite.extensions import SNIExtension, ClientCertTypeExtension, \
     SRPExtension, TLSExtension, NPNExtension
 from tlslite.errors import TLSInternalError
@@ -1965,6 +1965,33 @@ class TestServerKeyExchange(unittest.TestCase):
         self.assertEqual(ske.srp_s, bytearray(3))
         self.assertEqual(ske.srp_B, 4)
 
+    def test_parse_and_write_with_zero_padded_SRP(self):
+        ske = ServerKeyExchange(CipherSuite.TLS_SRP_SHA_WITH_AES_128_CBC_SHA,
+                                (3, 3))
+
+        msg = bytearray(
+            b'\x00\x00\x10' +       # overall length
+            b'\x00\x02' +           # N parameter length
+            b'\x00\x01' +               # N value
+            b'\x00\x02' +           # g parameter length
+            b'\x00\x02' +               # g value
+            b'\x03' +               # s parameter length
+            b'\x00'*3 +             # s value
+            b'\x00\x02' +           # B parameter length
+            b'\x00\x04'                 # B value
+            )
+
+        parser = Parser(msg)
+
+        ske.parse(parser)
+
+        self.assertEqual(ske.srp_N, 1)
+        self.assertEqual(ske.srp_g, 2)
+        self.assertEqual(ske.srp_s, bytearray(3))
+        self.assertEqual(ske.srp_B, 4)
+
+        self.assertEqual(ske.write()[1:], msg)
+
     def test_parser_with_SRP_RSA(self):
         ske = ServerKeyExchange(
                 CipherSuite.TLS_SRP_SHA_RSA_WITH_AES_128_CBC_SHA,
@@ -2042,6 +2069,29 @@ class TestServerKeyExchange(unittest.TestCase):
         self.assertEqual(ske.dh_g, 2)
         self.assertEqual(ske.dh_Ys, 16)
 
+    def test_parser_read_write_with_zero_padded_DH(self):
+        ske = ServerKeyExchange(CipherSuite.TLS_DH_ANON_WITH_AES_128_CBC_SHA,
+                                (3, 3))
+        msg = bytearray(
+            b'\x00\x00\x0c' +       # overall length
+            b'\x00\x02' +           # p parameter length
+            b'\x00\x1f' +               # p value
+            b'\x00\x02' +           # g parameter length
+            b'\x00\x02' +               # g value
+            b'\x00\x02' +           # Ys parameter length
+            b'\x00\x10'                 # Ys value
+            )
+
+        parser = Parser(msg)
+
+        ske.parse(parser)
+
+        self.assertEqual(ske.dh_p, 31)
+        self.assertEqual(ske.dh_g, 2)
+        self.assertEqual(ske.dh_Ys, 16)
+
+        self.assertEqual(ske.write()[1:], msg)
+
     def test_parser_with_ECDH(self):
         ske = ServerKeyExchange(CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
                                 (3, 3))
@@ -2362,5 +2412,47 @@ class TestServerFinished(unittest.TestCase):
         self.assertEqual(fin.verify_data, bytearray(b'\xc0\xfe'))
         self.assertEqual(fin.handshakeType, SSL2HandshakeType.server_finished)
 
+
+class TestCertificateStatus(unittest.TestCase):
+    def setUp(self):
+        self.msg = CertificateStatus()
+
+    def test___init__(self):
+        self.assertIsNotNone(self.msg)
+        self.assertEqual(self.msg.handshakeType,
+                         22)
+
+    def test_create(self):
+        m = self.msg.create(12, bytearray(b'\xab\xba'))
+        self.assertIs(m, self.msg)
+
+        self.assertEqual(m.status_type, 12)
+        self.assertEqual(m.ocsp, bytearray(b'\xab\xba'))
+
+    def test_parse(self):
+        parser = Parser(bytearray(
+            # type is handled by the dispatcher
+            b'\x00\x00\x05'  # overall length
+            b'\x10'  # type of the payload
+            b'\x00\x00\x01'  # patyload length
+            b'\xfa'))
+
+        m = self.msg.parse(parser)
+        self.assertIs(m, self.msg)
+
+        self.assertEqual(m.status_type, 16)
+        self.assertEqual(m.ocsp, bytearray(b'\xfa'))
+
+    def test_write(self):
+        self.msg.create(CertificateStatusType.ocsp, bytearray(b'\xbc\xaa'))
+
+        self.assertEqual(self.msg.write(), bytearray(
+            b'\x16'
+            b'\x00\x00\x06'
+            b'\x01'
+            b'\x00\x00\x02'
+            b'\xbc\xaa'))
+
+
 if __name__ == '__main__':
     unittest.main()
